<?php

namespace TDevAgency\GitlabLoggingHandler\Jobs;

use Gitlab\Client;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Support\Facades\Config;
use Throwable;

class GitlabLoggingJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable;

    private string $message;

    private array $formatted;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct(array $record)
    {
        if ($this->hasGitlabConfig()) {
            $this->message = $record['message'];
            $this->formatted = $record['formatted'];
        }
    }

    private function hasGitlabConfig(): bool
    {
        return ! is_null(Config::get('logging.channels.gitlab.credentials.url'))
            && ! is_null(Config::get('logging.channels.gitlab.credentials.project_id'))
            && ! is_null(Config::get('logging.channels.gitlab.credentials.token'));
    }

    /**
     * Execute the job.
     *
     * @return void
     * @throws Throwable
     */
    public function handle()
    {
        try {
            if (! $this->hasGitlabConfig()) {
                return;
            }
            $client = $this->client();
            $projectId = Config::get('logging.channels.gitlab.credentials.project_id');
            $searchResult = $client->issues()->all($projectId, ['search' => $this->message, 'state' => 'opened']);
            if (! empty($searchResult)) {
                return;
            }
            $title = $this->message;
            $description = '```json'
                .PHP_EOL
                .json_encode($this->formatted, JSON_THROW_ON_ERROR | JSON_UNESCAPED_SLASHES | JSON_PRETTY_PRINT)
                .PHP_EOL
                .'```';
            $client->issues()->create(
                $projectId,
                compact('title', 'description')
            );
        } catch (\Exception $exception) {
            // Ignore any exceptions there;
        }
    }

    private function client()
    {
        $client = new Client();
        $client->setUrl(Config::get('logging.channels.gitlab.credentials.url'));
        $client->authenticate(
            Config::get('logging.channels.gitlab.credentials.token'),
            Client::AUTH_HTTP_TOKEN
        );

        return $client;
    }
}
